#include <stdlib.h>
#include <wlr/backend.h>
#include <wlr/render/wlr_renderer.h>
#include <wlr/render/allocator.h>
#include <wlr/types/wlr_scene.h>
#include <wlr/types/wlr_output_layout.h>
#include <wlr/util/log.h>
#include <wlr/types/wlr_output.h>
#include <panes/renderer.h>
#include <panes/compositor.h>
#include <ctype.h>
#include <string.h>
#include <wlr/types/wlr_compositor.h>
#include <wlr/types/wlr_data_device.h>
#include <wlr/backend/headless.h>  // Add this to the top of the file

static bool parse_hex_color(const char *hex_str, float *rgba_out) {
    if (!hex_str) return false;

    if (hex_str[0] == '#') hex_str++;
    else if (hex_str[0] == '0' && tolower(hex_str[1]) == 'x') hex_str += 2;

    if (strlen(hex_str) != 6) return false;

    char r_str[3] = { hex_str[0], hex_str[1], '\0' };
    char g_str[3] = { hex_str[2], hex_str[3], '\0' };
    char b_str[3] = { hex_str[4], hex_str[5], '\0' };

    int r = strtol(r_str, NULL, 16);
    int g = strtol(g_str, NULL, 16);
    int b = strtol(b_str, NULL, 16);

    rgba_out[0] = r / 255.0f;
    rgba_out[1] = g / 255.0f;
    rgba_out[2] = b / 255.0f;
    rgba_out[3] = 1.0f;

    return true;
}

void compositor_init(struct compositor_state *state) {
    state->display = wl_display_create();
    struct wl_event_loop *loop = wl_display_get_event_loop(state->display);

    state->backend = wlr_backend_autocreate(loop, NULL);
    state->renderer = wlr_renderer_autocreate(state->backend);
    state->allocator = wlr_allocator_autocreate(state->backend, state->renderer);
    state->layout = wlr_output_layout_create(state->display);
    state->scene = wlr_scene_create();

    // Add headless output if applicable (nested testing)
    if (wlr_backend_is_headless(state->backend)) {
        struct wlr_output *headless_output = wlr_headless_add_output(state->backend, 800, 600);
        if (!headless_output) {
            wlr_log(WLR_ERROR, "Failed to create headless output");
        } else {
            struct wlr_output_state pending;
            memset(&pending, 0, sizeof(pending));
            pending.committed = WLR_OUTPUT_STATE_ENABLED | WLR_OUTPUT_STATE_MODE;
            pending.enabled = true;
            pending.mode_type = WLR_OUTPUT_STATE_MODE_CUSTOM;
            pending.custom_mode.width = 800;
            pending.custom_mode.height = 600;
            pending.custom_mode.refresh = 60000;

            if (!wlr_output_commit_state(headless_output, &pending)) {
                wlr_log(WLR_ERROR, "Failed to commit headless output state");
            }
        }
    }

    wlr_compositor_create(state->display, 4, state->renderer);
    wlr_data_device_manager_create(state->display);

    wl_signal_add(&state->backend->events.new_output, &state->new_output);
    state->new_output.notify = handle_new_output;
}

void handle_new_output(struct wl_listener *listener, void *data) {
    struct compositor_state *state = wl_container_of(listener, state, new_output);
    struct wlr_output *output = data;

    if (!wl_list_empty(&output->modes)) {
        struct wlr_output_mode *mode = wl_container_of(output->modes.next, mode, link);

        struct wlr_output_state pending;
        memset(&pending, 0, sizeof(pending));
        pending.committed = WLR_OUTPUT_STATE_ENABLED | WLR_OUTPUT_STATE_MODE;
        pending.enabled = true;
        pending.mode_type = WLR_OUTPUT_STATE_MODE_FIXED;
        pending.mode = mode;

        if (!wlr_output_commit_state(output, &pending)) {
            wlr_log(WLR_ERROR, "Failed to commit output");
            return;
        }
    }

    wlr_output_layout_add_auto(state->layout, output);
    wlr_scene_output_create(state->scene, output);
    renderer_set_background(state->scene, 0x018281); // Hex color
}
